# -*- coding: utf-8 -*-

from sqlalchemy import *
from dictionaries.versions import AlterBigTable
import re
try:
    from sqlalchemy.dialects.mssql.pyodbc import MSDialect_pyodbc
except:
    from sqlalchemy.databases.mssql import MSSQLDialect_pyodbc as MSDialect_pyodbc


def isInnoDB(engine, tableName):
    connection = engine.connect()
    version = connection.execute("SELECT engine FROM information_schema.tables WHERE table_schema='dictionaries' AND table_name='%s';" % (tableName))   
    for res in version:
        return str(res[0]) == "InnoDB"

def get_table_name(engine,name):
    conn = engine.connect()
    if type(conn.dialect) is not MSDialect_pyodbc:
        create_view = conn.execute('show create view %s' % name).fetchone()[1]
        m = re.match('.*(%s[12]).*' % name, create_view)
        return m.group(1)

def update_dic_view(engine, name, tblname = None):
    conn = engine.connect()
    if type(conn.dialect) is MSDialect_pyodbc:
        conn.connection.connection.execute("sp_refreshview '%s';" % (name))
        conn.connection.connection.commit()
    else:
        if tblname is None:
            try:
                create_view = conn.execute('show create view %s' % name).fetchone()[1]
            except Warning:
                pass
            m = re.match('.*(%s[12]).*' % name, create_view)
            conn.execute('alter view %s as select * from %s' % (name, m.group(1)))
        else:
            conn.execute('alter view %s as select * from %s' % (name, tblname))

def create_or_replace(engine,name, tablename):
    conn = engine.connect()
    if type(conn.dialect) is MSDialect_pyodbc:
        conn.connection.connection.execute("IF EXISTS (SELECT TABLE_NAME FROM INFORMATION_SCHEMA.VIEWS WHERE TABLE_NAME = '%s') DROP VIEW '%s';" % (name,name))
        conn.connection.connection.commit()
        conn.connection.connection.execute("CREATE VIEW %s AS SELECT * FROM %s" % (name, tablename))
        conn.connection.connection.commit()
    else:
        conn.execute('create or replace view %s as select * from %s' % (name, tablename))

def drop_if_exist(engine,name):
    conn = engine.connect()
    if type(conn.dialect) is MSDialect_pyodbc:
        conn.connection.connection.execute("IF EXISTS (SELECT TABLE_NAME FROM INFORMATION_SCHEMA.VIEWS WHERE TABLE_NAME = '%s') DROP VIEW '%s';" % (name,name))
        conn.connection.connection.commit()		
    else:
        conn.execute('drop view if exists %s' % (name))

def getTables(meta, table_name):
    '''
    Получения таблиц из dictionaries по имени
    table_name - наименование таблицы
    '''
    tbl1 = Table('%s1' % table_name, meta, autoload=True)
    tbl2 = Table('%s2' % table_name, meta, autoload=True)
    return [tbl1, tbl2]

def create_column(migrate_engine, table_name, column_name, column_type, isNullable=True, isBigTable=False, default=None):
    '''
    Добавление колонки в таблицу
    table_name - имя таблицы
    column_name - наименование колонки 
    column_type - тип данных колонки
    isNullable - True - если поле может принимать значение Null, False - иначе
    isBigTable - True - если поле добавляется в таблицу с большим объемом данных (card, client, tmc), False - иначе
    default - значение по умолчанию
    '''
    meta = MetaData(bind = migrate_engine)
    for tbl in getTables(meta, table_name):
        if not column_name in tbl.columns:
            column = Column(column_name, column_type, nullable=isNullable, default=default, quote=True)
            if isBigTable and not isInnoDB(migrate_engine, tbl.name):
                AlterBigTable.AlterTableAddColumn(migrate_engine, tbl, column)
            else:
                column.create(tbl)
    update_dic_view(migrate_engine, table_name)

def alter_column(migrate_engine, table_name, column_name, isNullable=True):
    '''
    Изменение колонки
    table_name - имя таблицы
    column_name - наименование колонки 
    isNullable - True - если поле может принимать значение Null, False - иначе
    '''
    meta = MetaData(bind=migrate_engine)
    for table in getTables(meta, table_name):
        if column_name in table.columns:
            getattr(table.c, column_name).alter(nullable=isNullable)
    update_dic_view(migrate_engine, table_name)

def drop_column(migrate_engine, table_name, column_name, column_type, isBigTable=False):
    '''
    Удаление колонки из таблицы по имени
    table_name - имя таблицы
    column_name - наименование колонки
    column_type - тип данных колонки
    isBigTable - True - если поле удаляется из таблицы с большим объемом данных (card, client), False - иначе
    '''
    meta = MetaData(bind = migrate_engine)

    table_name_view = get_table_name(migrate_engine, table_name)

    for tbl in getTables(meta, table_name):
        if column_name in tbl.columns:
            column = Column(column_name, column_type, quote=True)
            if isBigTable and not isInnoDB(migrate_engine, tbl.name):
                AlterBigTable.AlterTableDropColumn(migrate_engine, tbl, column, table_name_view)
            else:
                column.drop(tbl)
    update_dic_view(migrate_engine, table_name, table_name_view)

def create_index(migrate_engine, table_name, column_name, index_name, isUnique=False, isBigTable=False):
    '''
    Добавление индекса в таблицу
    table_name - имя таблицы
    column_name - наименование колонки 
    index_name - наименование индекса
    isUnique - True - если индекс уникальный
    isBigTable - True - если поле добавляется в таблицу с большим объемом данных (card, client), False - иначе
    '''
    meta = MetaData(bind = migrate_engine)
    for tbl in getTables(meta, table_name):
        if index_name not in [ind.name for ind in tbl.indexes]:
            if column_name in tbl.columns:
                if isBigTable and not isInnoDB(migrate_engine, tbl.name):
                    AlterBigTable.AlterTableAddIndex(migrate_engine, tbl, column_name, index_name)
                else:
                    for column in tbl.columns:
                        if column_name == column.name:
                            Index(index_name, column, unique=isUnique).create()
                            break

    update_dic_view(migrate_engine, table_name)

def drop_index(migrate_engine, table_name, index_name, isBigTable=False):
    '''
    Удаление индекса из таблицы
    table_name - имя таблицы
    index_name - наименование индекса
    isBigTable - True - если поле удаляется из таблицы с большим объемом данных (card, client), False - иначе
    '''
    meta = MetaData(bind = migrate_engine)

    table_name_view = get_table_name(migrate_engine, table_name)
    for tbl in getTables(meta, table_name):
        if index_name in [ind.name for ind in tbl.indexes]:
            if isBigTable and not isInnoDB(migrate_engine, tbl.name):
                AlterBigTable.AlterTableDropIndex(migrate_engine, tbl, index_name, table_name_view)
            else:
                for index in tbl.indexes:
                    if index_name == index.name:
                        index.drop()
                        break
    update_dic_view(migrate_engine, table_name, table_name_view)

def create_table(migrate_engine, table):
    '''
    Создание новой таблицы
    table объект таблицы sqlalchemy.Table
    '''
    basename = table.name
    for suffix in ['1','2']:
        table.name = basename + suffix
        if not table.exists():
            table.create()
    create_or_replace(migrate_engine, basename, basename + '1')

def drop_table(migrate_engine, table):
    '''
    Удаление таблицы
    table объект таблицы sqlalchemy.Table
    '''
    basename = table.name
    for suffix in ['1','2']:
        table.name = basename + suffix
        if table.exists():
            table.drop()
    drop_if_exist(migrate_engine, basename)


def isColumnExists(migrate_engine, table, column):
    '''
    Проверка существования колонки в таблице
    table название таблицы
    column название колонки
    '''
    meta = MetaData(bind = migrate_engine)

    result = True
    for tbl in getTables(meta, table):
        result = result and (column in tbl.columns)

    return result
